﻿using Dapper;
using DocumentFormat.OpenXml.Bibliography;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using PIbd_24_EredavkinRA_BusBusiness.Entities;


namespace PIbd_24_EredavkinRA_BusBusiness.Repositories.Implementations;

public class RoutSheetRepository : IRoutSheetRepository
{
    private readonly IConnectionString _connectionString;

    private readonly ILogger<RoutSheetRepository> _logger;

    public RoutSheetRepository(IConnectionString connectionString, ILogger<RoutSheetRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }
    public void CreateRoutSheet(RoutSheet routsheet)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(routsheet));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            connection.Open();
            using var transaction = connection.BeginTransaction();
            var queryInsert = @"
                INSERT INTO RoutSheet (RoutId, DriverId, ConductorId, BusDate)
                VALUES (@RoutId, @DriverId, @ConductorID, @BusDate);
                SELECT MAX(Id) FROM RoutSheet";
            var routsheetId = connection.QueryFirst<int>(queryInsert,  routsheet, transaction);

            var querySubInsert = @"
                INSERT INTO RoutSheetBus (RoutSheetId, BusId)
                VALUES (@RoutSheetId, @BusId)";
            foreach (var elem in routsheet.RoutSheetBus)
            {
                connection.Execute(querySubInsert, new { routsheetId, elem.BusID }, transaction);
            }

            transaction.Commit();
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }

   

    public IEnumerable<RoutSheet> ReadRoutSheet(DateTime? dateForm = null, DateTime? dateBus = null, int? routId = null, int? busid = null, int? driverid = null, int? conductorid = null)
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            var builder = new QueryBuilder();
            if (dateForm.HasValue)
            {
                builder.AddCondition("cs.BusDate >= @dateForm");
            }
            if (dateBus.HasValue)
            {
                builder.AddCondition("cs.BusDate <= @dateBus");
            }
            if (driverid.HasValue)
            {
                builder.AddCondition("cs.DriverId = @driverid");
            }
            if (conductorid.HasValue)
            {
                builder.AddCondition("cs.ConductorId = @conductorid");
            }
            if (busid.HasValue)
            {
                builder.AddCondition("rsh.BusId = @busid");
            }
            if (routId.HasValue)
            {
                builder.AddCondition("cs.RoutId = @routid");
            }

            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = $@"
                SELECT
                    cs.*,
                    CONCAT(s.FirstName, ' ', s.LastName) AS EmployeeNameDriver,
                    CONCAT(c.FirstName, ' ', c.LastName) AS EmployeeNameConductor,
                    CONCAT(ro.NumberRout, ' ', ro.StartStop, ' ', ro.EndStop) AS RoutName,
                    rsh.BusId,
                    b.N_Z AS BusName
                FROM RoutSheet cs
                LEFT JOIN Staff s ON s.Id = cs.DriverId
                LEFT JOIN Staff c ON c.Id = cs.ConductorId
                LEFT JOIN Rout ro ON ro.Id = cs.RoutId
                INNER JOIN RoutSheetBus rsh ON rsh.RoutSheetId = cs.Id
                LEFT JOIN Bus b ON b.Id = rsh.BusId
                {builder.Build()}";
            var supplementDict = new Dictionary<int, List<RoutSheetBus>>();

            var curriculumSupplements = connection.Query<RoutSheet, RoutSheetBus, RoutSheet>(querySelect,
                (supplement, curriculumSupplements) =>
                {
                    if (!supplementDict.TryGetValue(supplement.Id, out var rsh))
                    {
                        rsh = [];
                        supplementDict.Add(supplement.Id, rsh);
                    }

                    rsh.Add(curriculumSupplements);
                    return supplement;
                }, splitOn: "BusId", param: new { dateForm, dateBus, routId, busid, driverid, conductorid });
            _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(curriculumSupplements));

            return supplementDict.Select(x =>
            {
                var cs = curriculumSupplements.First(y => y.Id == x.Key);
                cs.SetRoutSheetBus(x.Value);
                return cs;
            }).ToArray();
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }
}
